/***************************************************************************
 *   Copyright (C) 2007 by Balázs Béla                                     *
 *   balazsbela@gmail.com                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation version 2.                               *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/*
 Special thanks to: Xavier Corredor Llano, the creator of the nuvoX iconset which my icons are mostly based on.I would like to thank him for releasing this great iconset under GPL.
*/

/* 
   For the interface we use a central QStackedWidget object named mainWid and we change the 
   currently active page with the help of the toolbar.
*/


#include <QtGui>
#include "qorganizer.h"
#include <QCloseEvent>
#include <QFileDialog>
#include <cstdlib>
#include <math.h>

using namespace std;

//The constructor
qOrganizer::qOrganizer()
{ 
      setWindowTitle("qOrganizer");   
      QSize size(1024,768);
      resize(size);
      readSettings();
      setWindowIcon(QIcon(":/images/icon.png"));
      readConfigFile();
      addItems();
      createActions();
      createMenus();
      createToolBars();
      createStatusBar();     
}

void qOrganizer::readSettings()
{
  QSettings settings("qOrganizer", "qOrganizer");
  QSize size = settings.value("windowsize").toSize();
  this -> resize(size);
  bool maximized = settings.value("windowmaximized").toBool();
  if(maximized) this->showMaximized();
  
  //FTP settings:
  C_HOST = settings.value("host").toString();
  C_PORT = settings.value("port").toInt();
  C_USER = settings.value("user").toString();
  C_PASSWD = settings.value("passwd").toString();
  C_FTPPATH = settings.value("path").toString();
  //Other settings:
  C_SYSTRAY = settings.value("tosystrayonclose").toBool();
  C_SOUND = settings.value("soundonremind").toBool();
};

void qOrganizer::writeSettings()
{
 QSettings settings("qOrganizer", "qOrganizer");
 settings.setValue("windowsize", this->size());
 settings.setValue("windowmaximized", this->isMaximized());
};

//The about box
void qOrganizer::about()
{
      QMessageBox::about(this, tr("About qOrganizer")+" v2.0",
            tr("<b>qOrganizer : Copyright 2007 Bal&aacute;zs B&eacute;la<br>\n"
               "\nE-mail:balazsbela@gmail.com<br>\n"
               "\n"
               "\n<img src=:/images/logo.png><br>\n"
               "\n"
               "\n<br>This software is licenced under GPL version 2 published by"
               " the Free Software Foundation.<br>\n" 
               "\n"
               "<br>This application aims to bring simplicity in your life by "
               " organizing your data in an intuitive way so you can " "always find what you need and never forget something " "important.<br>This application focuses on students, providing "
               "the features they consider usefull.<br><br> \n"
               "Special thanks to:my girlfriend for " "supporting me, to my friends for testing this " "application,"
               "and to Xavier Corredor Llano, for releasing the nuvoX " "iconset under GPL</b>"
              ));
}

//Actions
void qOrganizer::createActions()
{
      //calendar Action
      calAct = new QAction(QIcon(":/images/calendar.xpm"), tr("&Calendar"), this);
      calAct->setShortcut(tr("Ctrl+Alt+C"));
      calAct->setStatusTip(tr("Calendar"));
      calAct->setToolTip(tr("Calendar"));
      connect(calAct, SIGNAL(triggered()),this, SLOT(ChangeToCalendar())); 
      
      //save Action
      saveAct = new QAction(QIcon(":/images/save.xpm"),tr("Save"),this);
      saveAct->setShortcut(tr("Ctrl+S"));
      saveAct->setStatusTip(tr("Save"));
      saveAct->setToolTip(tr("Save"));  
      connect(saveAct,SIGNAL(triggered()),this, SLOT(saveSchedule()));
      connect(saveAct,SIGNAL(triggered()),this, SLOT(saveJurnal()));
      connect(saveAct,SIGNAL(triggered()),this, SLOT(saveToDoList()));
      connect(saveAct,SIGNAL(triggered()),this, SLOT(saveTimeTable()));
      connect(saveAct,SIGNAL(triggered()),this, SLOT(saveMarksTable()));
      connect(saveAct,SIGNAL(triggered()),this, SLOT(saveAbsenceTable()));
      
      //The action that changes to the to-do page
      toDoAct = new QAction(QIcon(":/images/todo.xpm"), tr("&To-do List"), this);
      toDoAct->setShortcut(tr("Ctrl+Alt+T"));
      toDoAct->setStatusTip(tr("Edit your to-do list"));
      toDoAct->setToolTip(tr("To-do list"));
      connect(toDoAct, SIGNAL(triggered()),this, SLOT(ChangetoToDoList())); 
      
      //The action that changes to the timetable page
      timeTableAct = new QAction(QIcon(":/images/timetable.xpm"), tr("&TimeTable"), this);
      timeTableAct->setShortcut(tr("Ctrl+Alt+L"));
      timeTableAct->setStatusTip(tr("Edit your timetable"));
      timeTableAct->setToolTip(tr("Timetable"));
      connect(timeTableAct, SIGNAL(triggered()),this, SLOT(ChangeToTimeTable())); 
 
      //The action that changes to the booklet actions
      bookletAct = new QAction(QIcon(":/images/booklet.xpm"), tr("&Booklet"), this);
      bookletAct->setShortcut(tr("Ctrl+Alt+B"));
      bookletAct->setStatusTip(tr("Keep track of your school marks and absences"));
      bookletAct->setToolTip(tr("Booklet"));
      connect(bookletAct, SIGNAL(triggered()),this, SLOT(ChangeToBooklet())); 

      //Print action
      printAct = new QAction(QIcon(":/images/print.xpm"), tr("&Print"), this);
      printAct->setShortcut(tr("Ctrl+P"));
      printAct->setStatusTip(tr("Print Page"));
      printAct->setToolTip(tr("Print Page"));
      connect(printAct, SIGNAL(triggered()),this, SLOT(printPage())); 

 
      //Exit action
      exitAct = new QAction(QIcon(":/images/exit.xpm"),tr("E&xit"), this);
      exitAct->setShortcut(tr("Ctrl+Q"));
      exitAct->setStatusTip(tr("Exit the application"));
      connect(exitAct, SIGNAL(triggered()), this, SLOT(exitApp()));

      //The action for the about box
      aboutAct = new QAction(QIcon(":/images/icon.xpm"),tr("&About"), this);
      aboutAct->setStatusTip(tr("Show the application's About box"));
      connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

      //About Qt
      aboutQtAct = new QAction(tr("About &Qt"), this);
      aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
      connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
        
      //Upload to FTP server
      uploadAct = new QAction(QIcon(":/images/upload.xpm"), tr("&Upload"), this);
      uploadAct ->setShortcut(tr("Ctrl+Alt+U"));
      uploadAct ->setStatusTip(tr("Upload data to FTP server"));
      uploadAct ->setToolTip(tr("Upload data to FTP server"));
      connect(uploadAct,SIGNAL(triggered()),this,SLOT(putToFTP()));
      
      //Download from FTP server
      downloadAct = new QAction(QIcon(":/images/download.xpm"), tr("&Download"), this);
      downloadAct ->setShortcut(tr("Ctrl+Alt+W"));
      downloadAct ->setStatusTip(tr("Download data from FTP server"));
      downloadAct ->setToolTip(tr("Download data from FTP server"));
      connect(downloadAct,SIGNAL(triggered()),this,SLOT(getFromFTP()));
      
      //Settings dialog
      settingsAct = new QAction(QIcon(":/images/settings.xpm"),tr("&Settings"),this);
      settingsAct -> setShortcut(tr("Ctrl+Alt+S"));
      settingsAct -> setStatusTip(tr("Configure application"));
      connect(settingsAct, SIGNAL(triggered()), this ,SLOT(showSettings()));
      
}


//Set up menus
void qOrganizer::createMenus()
{
      fileMenu = menuBar()->addMenu(tr("&File"));
      fileMenu->addAction(saveAct);
      fileMenu->addAction(printAct);
      fileMenu->addAction(exitAct);

      viewMenu = menuBar()->addMenu(tr("&View"));
      viewMenu -> addAction(calAct);
      viewMenu -> addAction(toDoAct);
      viewMenu -> addAction(timeTableAct);
      viewMenu -> addAction(bookletAct);
    
      settingsMenu = menuBar()->addMenu(tr("&Settings"));      
      settingsMenu -> addAction(settingsAct);

      ftpMenu = menuBar() -> addMenu(tr("&FTP"));
      ftpMenu -> addAction(uploadAct);
      ftpMenu -> addAction(downloadAct);
 
      helpMenu = menuBar()->addMenu(tr("&Help"));
      helpMenu->addAction(aboutAct);
      //helpMenu->addAction(aboutQtAct);
     
      contextMenu = new QMenu();
      contextMenu -> addAction(settingsAct);
      contextMenu -> addAction(aboutAct);
      contextMenu -> addAction(exitAct);
      tray -> setContextMenu(contextMenu);
}

//Set up toolbars
void qOrganizer::createToolBars()
{
     
      fileToolBar = addToolBar(tr("File")); 
      fileToolBar->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
      fileToolBar->setIconSize(QSize(64,64));
      //fileToolBar->setContentsMargins(50,0,100,0);
      fileToolBar->addAction(calAct);
      fileToolBar->addAction(toDoAct);
      fileToolBar->addAction(timeTableAct);
      fileToolBar->addAction(bookletAct); 
      fileToolBar->addAction(printAct);
      if(!C_AUTOSAVE_TOGGLE)
       fileToolBar->addAction(saveAct);
      if(C_SYNCFTP)
       { 
        fileToolBar->addAction(downloadAct);
        fileToolBar->addAction(uploadAct);        
       }
      fileToolBar -> addAction(settingsAct);
      
}

void qOrganizer::createStatusBar()
{
      statusBar()->showMessage(tr("Ready"));
      statusProgress = new QProgressBar(statusBar());
      statusBar() -> addPermanentWidget(statusProgress,1);
      statusProgress -> setMaximumHeight(20);
      statusProgress -> setMaximumWidth(120);
      statusProgress -> setMinimum(0);
      statusProgress -> setMaximum(0);
      statusProgress -> hide();
     
}

void qOrganizer::addCalendarPageWidgets()
{
  //widgets in the page 
  calendar=new QCalendarWidget; //The Calendar
  calendar->setGridVisible(true);
  calendar-> setFirstDayOfWeek (Qt::DayOfWeek(C_FIRST_DAY_OF_WEEK)); 
  calendar->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Fixed);
  
  QDate date = calendar->selectedDate();
  selDate = date.toString(C_DT_FORMAT);
  QString Date = tr("<center><font size=\"5\">Schedule for ")+selDate+tr(":</font></center>");
  day = new QLabel(Date);

  connect(calendar, SIGNAL(selectionChanged()), this, SLOT(updateDay()));
  
  tableWid = new CQTableWidget();
  tableWid->setRowCount(C_NRROWS);
  tableWid->setColumnCount(4);
  tableWid->setParent(this);
  tableWid -> verticalHeader()->hide();
//  tableWid -> setSortingEnabled(true);
  tableWid -> setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
  
  Clabels << tr("Event") << tr("From")<< tr("Until")<<tr("Reminder");
  
  
  tableWid ->setHorizontalHeaderLabels(Clabels);
  tableWid -> setColumnWidth(0,285);
  tableWid -> setColumnWidth(1,60);
  tableWid -> setColumnWidth(2,60);
  tableWid -> setColumnWidth(3,75);  

  tableWid -> horizontalHeader() -> setStretchLastSection(true);
  tableWid -> horizontalHeader() -> setMovable(true);
  tableWid -> setItemDelegate(&schDelegate);
  tableWid -> setObjectName("tableWid");

  newButton = new QPushButton(tr("New event"));
  newButton -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
  connect(newButton, SIGNAL(clicked()),this, SLOT(insertRowToEnd()));
 
  deleteButton = new QPushButton(tr("Delete event"));
  deleteButton -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
  connect(deleteButton, SIGNAL(clicked()),this, SLOT(deleteRow()));
 
  textField = new CQTextEdit;
  fontBox = new QFontComboBox;
  comboSize = new QComboBox();
  comboSize->setObjectName("comboSize");
  comboSize->setEditable(true);
  //Populate the font combo box
  QFontDatabase db;
  foreach(int size, db.standardSizes())
   comboSize->addItem(QString::number(size)); 
  //Set it to 12
  comboSize->setCurrentIndex(6);
 
  B=new QPushButton(tr("B"));
  I=new QPushButton(tr("I"));
  U=new QPushButton(tr("U"));
  
  B->setCheckable(true);
  U->setCheckable(true);
  I->setCheckable(true);
 
  B -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
  I -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
  U -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);

  //Set the selected Font
  this -> setFont();  
  
  QString note = tr("<font size=\"5\">Important notes, journal for ")+selDate+tr(":</font>");
  noteLabel = new QLabel(note); 

};

void qOrganizer::connectCallendarPageWidgets()
{
  //Make sure that the new Font is set if the boxes change
  connect(fontBox,SIGNAL(currentFontChanged(QFont)), this, SLOT(setFont()));
  connect(comboSize,SIGNAL(currentIndexChanged(int)), this, SLOT(setFont()));

  connect(B,SIGNAL(clicked()), this, SLOT(setFont()));
  connect(U,SIGNAL(clicked()), this, SLOT(setFont()));
  connect(I,SIGNAL(clicked()), this, SLOT(setFont())); 
  
  //Change the font for every new character
  connect(textField, SIGNAL(textChanged()), this, SLOT(modifyText())); 
  //Update if calendar changed 
  connect(calendar, SIGNAL(selectionChanged()), this, SLOT(loadSchedule()));
  connect(calendar, SIGNAL(selectionChanged()), this, SLOT(loadJurnal()));
  //Automatically Save after every change :P
  if(C_AUTOSAVE_TOGGLE)
   connect(tableWid,SIGNAL(focusLost()),this,SLOT(saveSchedule()));
  //for error checking call remind if cell changed
   connect(tableWid,SIGNAL(cellChanged(int,int)),this,SLOT(remind()));
  
  if(C_AUTOSAVE_TOGGLE)
   connect(textField,SIGNAL(focusLost()),this,SLOT(saveJurnal()));
  //If an event gets deleted save
  if(C_AUTOSAVE_TOGGLE)
  connect(deleteButton, SIGNAL(clicked()), this , SLOT(saveSchedule()));
 
};

void qOrganizer::layoutCalendarPageWidgets()
{
 //The layouts for the widgets
  
  buttonLayout = new QHBoxLayout;
  buttonLayout -> addWidget(newButton);
  buttonLayout -> addWidget(deleteButton);
  
  vl = new QVBoxLayout; //vertical layout
  vl -> addWidget(calendar);
  vl -> addWidget(day);
  vl -> addWidget(tableWid);
  vl -> addLayout(buttonLayout);
  vl -> setSizeConstraint(QLayout::SetNoConstraint); 
  
  QHBoxLayout *settingsLayout = new QHBoxLayout; 
  settingsLayout ->addWidget(fontBox);
  settingsLayout ->addWidget(comboSize);
  settingsLayout ->addWidget(B);
  settingsLayout ->addWidget(I);
  settingsLayout ->addWidget(U);
  settingsLayout ->setSizeConstraint(QLayout::SetNoConstraint);
  
  textLayout = new QVBoxLayout;
  textLayout ->addWidget(noteLabel);
  textLayout ->addLayout(settingsLayout);
  textLayout ->addWidget(textField);

  mainLayout = new QHBoxLayout;;
  mainLayout ->addLayout(vl);
  mainLayout ->addLayout(textLayout);

  CalendarPage= new QWidget; //Page that contains the callendar 
  CalendarPage->setLayout(mainLayout);

  mainWid->addWidget(CalendarPage);
  mainWid->setCurrentIndex(0);
  
};



void qOrganizer::addCalendarPage()
{
  
  addCalendarPageWidgets();
  loadJurnal();
  loadSchedule();
  connectCallendarPageWidgets();
  layoutCalendarPageWidgets();
  timer = new QTimer();
  setReminders();
};

//---------------------- TO-DO PAGE -----------------------------------

void qOrganizer::addToDoPage()
{
 
 //Widgets:

  ToDoPage=new QWidget;
  QLabel *ToDoTitle= new QLabel (tr("<center><font size=\"5\" >General to-do list:</center>"));
  
  list=new CQTableWidget;
  list->setRowCount(10);
  list->setColumnCount(5);
  list->setParent(this);
  list -> verticalHeader()->hide();
  list -> setObjectName("list");
 
  TDlabels <<tr("Start")<< tr("Task") << tr("Deadline") << tr("Priority")<<tr("Completed");
  
  list ->setHorizontalHeaderLabels(TDlabels);
  list -> setColumnWidth(0,80);
  list -> setColumnWidth(1,560);
  list -> setColumnWidth(2,80);
  list -> setColumnWidth(3,80);
  list -> setColumnWidth(4,90);
  list -> horizontalHeader() -> setStretchLastSection(true);
  list -> horizontalHeader() -> setMovable(true);
  list -> setItemDelegate(&tdlistDelegate);

  newTaskButton = new QPushButton(tr("New task"));
  deleteTaskButton = new QPushButton(tr("Delete task"));
  newTaskButton -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
  deleteTaskButton-> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
  connect(newTaskButton,SIGNAL(clicked()),this,SLOT(newTask()));
  connect(deleteTaskButton,SIGNAL(clicked()),this,SLOT(deleteTask()));
  if(C_AUTOSAVE_TOGGLE)
  connect(list,SIGNAL(focusLost()),this,SLOT(saveToDoList()));

 //Layouts:

  ToDoVl=new QVBoxLayout;
  ToDoVl->addWidget(ToDoTitle);
  ToDoVl->addWidget(list); 
  ToDoVl -> setSizeConstraint(QLayout::SetNoConstraint);

  buttonVl=new QHBoxLayout;
  buttonVl->addWidget(newTaskButton);
  buttonVl->addWidget(deleteTaskButton);
 
  mainHl = new QVBoxLayout;
  mainHl ->addLayout(ToDoVl);
  mainHl ->addLayout(buttonVl);
 
  QHBoxLayout *layout= new QHBoxLayout;
  layout->addSpacing(58);
  layout->addLayout(mainHl);
  layout->addSpacing(60);

  ToDoPage->setLayout(layout);
  mainWid->addWidget(ToDoPage);

};

//----------------------------Timetable----------------------------------

void qOrganizer::addTimeTable()
{
  timeTablePage=new QWidget;
  QLabel *timeTableTitle= new QLabel (tr("<center><font size=\"5\">Timetable:</font></center>"));
  
  table=new CQTableWidget;
  table->setRowCount(9);
  table->setColumnCount(7);
  table->setParent(this);
  //table -> verticalHeader()->hide();
  table -> setSortingEnabled(false);
  table -> setItemDelegate(&ttDelegate);
  
  TTlabels << tr("From") <<tr("Until") << tr("Monday") << tr("Tuesday") << tr("Wednesday")<<tr("Thursday")<<tr("Friday");
  
  //If no labels are loaded from file then these labels are written to it and it is retranslated
  // to the current language.
  universalTTlabels<<"From"<<"Until"<<"Monday"<<"Tuesday"<<"Wednesday"<<"Thursday"<<"Friday";
 
  table ->setHorizontalHeaderLabels(TTlabels);
  for(int i=0;i<=2;i++)
  {
   table -> setColumnWidth(i,55);
  };
  for(int i=2;i<=table->columnCount()-1;i++)
  {
   table -> setColumnWidth(i,900/(table->columnCount()-1));
  }
  table -> horizontalHeader() -> setStretchLastSection(true);

  newTTRow = new QPushButton(tr("New row"));
  newTTCmn = new QPushButton(tr("New column"));
  delTTCmn = new QPushButton(tr("Delete column"));
  delTTRow = new QPushButton(tr("Delete row"));
  newTTRow -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
  newTTCmn -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
  delTTRow-> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
  delTTCmn-> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
  connect(newTTRow,SIGNAL(clicked()),this,SLOT(newRow()));
  connect(delTTRow,SIGNAL(clicked()),this,SLOT(delRow()));
  connect(newTTCmn,SIGNAL(clicked()),this,SLOT(newColumn()));
  connect(delTTCmn,SIGNAL(clicked()),this,SLOT(delColumn()));
  if(C_AUTOSAVE_TOGGLE)
  connect(table,SIGNAL(focusLost()),this,SLOT(saveTimeTable()));

 //Layouts:

  Vl=new QVBoxLayout;
  Vl->addWidget(timeTableTitle);
  Vl->addWidget(table); 
  Vl -> setSizeConstraint(QLayout::SetNoConstraint);

  BHl=new QHBoxLayout;
  BHl->addWidget(newTTCmn);
  BHl->addWidget(delTTCmn);
  BHl->addWidget(newTTRow);
  BHl->addWidget(delTTRow);
  Vl->addLayout(BHl);
 
  QHBoxLayout *layout= new QHBoxLayout;
  layout->addSpacing(58);
  layout->addLayout(Vl);
  layout->addSpacing(60);

  timeTablePage->setLayout(layout);
  mainWid->addWidget(timeTablePage);
};

//-----------------------------Booklet----------------------------


void qOrganizer::addBookletPage()
{
 bookletPage=new QWidget();
 //Widgets
 QLabel *bLabel=new QLabel(tr("<center><font size=\"5\">Booklet</font></center>"));
 QLabel *markLabel = new QLabel(tr("<center>Marks:</center>"));
 QLabel *absenceLabel = new QLabel(tr("<center>Absences:</center>"));
 markTable = new CQTableWidget();
 absenceTable = new CQTableWidget();
 markTable -> setRowCount(8);
 markTable -> setColumnCount(6);
 markTable -> setParent(this);
 markTable -> horizontalHeader()->hide();
 
 
 for(int i=0;i<=6;i++)
 {
  markTable -> setColumnWidth(i,50);
 }; 
 
 subjectList <<tr("English")<<tr("Math")<<tr("IT")<<tr("History")<<tr("Geography")<<tr("Biology")<<tr("Physics")<<tr("Chemistry");
 markTable -> setVerticalHeaderLabels(subjectList);
 
 absenceTable -> setRowCount(8);
 absenceTable -> setColumnCount(5);
 absenceTable -> setParent(this);
 absenceTable -> horizontalHeader()->hide();
 absenceTable -> verticalHeader()->hide();
 absenceTable -> setItemDelegate(&absDelegate);
 
 for(int i=0;i<=absenceTable->columnCount();i++)
 {
  absenceTable -> setColumnWidth(i,630/absenceTable->columnCount());
 };  
 
 //Neded to delete cells in this, by the custom delegate
 absenceTable->setObjectName(QString("absenceTable"));

 newMarkButton = new QPushButton(tr("New mark column"));
 newSubjectButton = new QPushButton(tr("New subject"));
 deleteSubjectButton = new QPushButton(tr("Delete subject"));
 deleteMarkButton = new QPushButton(tr("Delete mark column"));
 newAbsenceButton = new QPushButton(tr("New absence column"));
 deleteAbsenceButton = new QPushButton(tr("Delete absence column"));
  
 newMarkButton -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
 newSubjectButton -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
 deleteMarkButton -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
 deleteSubjectButton -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
 newAbsenceButton -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
 deleteAbsenceButton -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed); 
 
 
 QLabel *averageFieldLabel = new QLabel(tr("Subject average:"));
 QLabel *absenceFieldLabel = new QLabel(tr("Absences:"));
 averageField = new QLineEdit();
 absenceNrField = new QLineEdit();
 averageField -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed); 
 absenceNrField -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed); 
 
 totalAverageField = new QLineEdit();
 totalAverageField -> setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed); 
 QLabel *totalAverageLabel = new QLabel(tr("Total average:"));
 QLabel *voidLabel = new QLabel("  ");
 //Layouts 
 
 QHBoxLayout *averageLayout = new QHBoxLayout;
 averageLayout -> addWidget(averageFieldLabel); 
 averageLayout -> addWidget(averageField);
 averageLayout -> addWidget(voidLabel);
 averageLayout -> addWidget(totalAverageLabel);
 averageLayout -> addWidget(totalAverageField);

 QHBoxLayout *absenceFieldLayout = new QHBoxLayout;
 absenceFieldLayout -> addWidget(absenceFieldLabel); 
 absenceFieldLayout -> addWidget(absenceNrField);

 QVBoxLayout *markLayout = new QVBoxLayout;
 markLayout -> addWidget(markLabel);
 markLayout -> addWidget(markTable);
 markLayout -> addLayout(averageLayout); 

 QVBoxLayout *absenceLayout = new QVBoxLayout;
 absenceLayout -> addWidget(absenceLabel);
 absenceLayout -> addWidget(absenceTable);
 absenceLayout -> addLayout(absenceFieldLayout);
 

 QHBoxLayout *tableLayout = new QHBoxLayout;
 tableLayout -> addLayout(markLayout);
 tableLayout -> addSpacing(30);
 tableLayout -> addLayout(absenceLayout);

 QHBoxLayout *buttonLayout = new QHBoxLayout;
 buttonLayout -> addWidget(newMarkButton); 
 buttonLayout -> addWidget(deleteMarkButton);
 buttonLayout -> addWidget(newSubjectButton);
 buttonLayout -> addWidget(deleteSubjectButton);
 buttonLayout -> addWidget(newAbsenceButton);
 buttonLayout -> addWidget(deleteAbsenceButton);
 
 QVBoxLayout *mainLayout = new QVBoxLayout;
 mainLayout -> addWidget(bLabel);
 mainLayout -> addLayout(tableLayout); 
 mainLayout -> addLayout(buttonLayout);
 
 bookletPage->setLayout(mainLayout);
 mainWid->addWidget(bookletPage);

 //Connect buttons 
 loadMarksTable();
 loadAbsenceTable();
 markTable->setCurrentCell(0,0);
 updateAverage(0,0,1,0);
 updateAbsenceNr();
 updateTotalAverage();

 connect(newSubjectButton,SIGNAL(clicked()),this,SLOT(newSubject()));
 connect(deleteSubjectButton,SIGNAL(clicked()),this,SLOT(deleteSubject()));
 connect(newMarkButton,SIGNAL(clicked()),this,SLOT(newMark()));
 connect(deleteMarkButton,SIGNAL(clicked()),this,SLOT(deleteMark()));
 connect(newAbsenceButton,SIGNAL(clicked()),this,SLOT(newAbsence()));
 connect(deleteAbsenceButton,SIGNAL(clicked()),this,SLOT(deleteAbsence()));
 if(C_AUTOSAVE_TOGGLE)
 connect(markTable,SIGNAL(focusLost()),this,SLOT(saveMarksTable()));
 if(C_AUTOSAVE_TOGGLE)
 connect(absenceTable,SIGNAL(focusLost()),this,SLOT(saveAbsenceTable()));
 
 connect(markTable,SIGNAL(currentCellChanged(int,int,int,int)),this,SLOT(updateAverage(int,int,int,int)));
 connect(markTable,SIGNAL(cellChanged(int,int)),this,SLOT(updateTotalAverage()));
 connect(absenceTable,SIGNAL(cellChanged(int,int)),this,SLOT(updateAbsenceNr()));

};


// Let's put these all together 

void qOrganizer::addItems()
{
  QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF8"));
  QTextCodec::setCodecForTr(QTextCodec::codecForName("UTF8"));
  setSTray();
  mainWid= new QStackedWidget; //Let's set up the Main Widget
  setCentralWidget(mainWid);  //Let's make it the Central Widget
  addCalendarPage();
  addToDoPage();  
  addTimeTable();
  addBookletPage();
  loadToDoList();
  loadTimeTable();
  //Resize timetable
  for(int i=2;i<=table->columnCount()-1;i++)
   table -> setColumnWidth(i,900/(table->columnCount()-1));
  
};

//-----------------------SLOTS---------------------------------

/*This function sets the QStackedWidgets current index to 0 therefore makes the 
calendar page active*/

void qOrganizer::ChangeToCalendar()
{
 if(C_LOAD_ON_CHANGE) 
 {
  loadSchedule();
  loadJurnal();
 }
 if(C_AUTOSAVE_TOGGLE) saveAll();
 mainWid->setCurrentIndex(0);
};

/*This function sets the QStackedWidgets current index to 1 therefore makes the 
to-do list page active*/

void qOrganizer::ChangetoToDoList()
{ 
 if(C_LOAD_ON_CHANGE) loadToDoList();
 if(C_AUTOSAVE_TOGGLE) saveAll();
 mainWid->setCurrentIndex(1);
};

/*This function sets the QStackedWidgets current index to 2 therefore makes the 
timetable page active*/
 
void qOrganizer::ChangeToTimeTable()
{ 
 if(C_LOAD_ON_CHANGE) loadTimeTable();
 if(C_AUTOSAVE_TOGGLE) saveAll();
 mainWid->setCurrentIndex(2);
};


/*This function sets the QStackedWidgets current index to 3 therefore makes the 
Booklet page active*/

void qOrganizer::ChangeToBooklet()
{
 if(C_LOAD_ON_CHANGE)
 {
  loadMarksTable();
  loadAbsenceTable();
 }
 if(C_AUTOSAVE_TOGGLE) saveAll();
 mainWid->setCurrentIndex(3);
};

//-----------------------CALENDAR SLOTS----------------------------

//adds a new row to the schedule table, which is named tableWid (from tableWidget)
void qOrganizer::insertRowToEnd()
{
 tableWid->setRowCount(tableWid->rowCount()+1);
}

//deletes a row from the schedule table
void qOrganizer::deleteRow()
{
 tableWid->removeRow(tableWid->currentRow());
};

//Updates the two labels when the callendar changes date
void qOrganizer::updateDay()
{
  QDate date = calendar->selectedDate();
  selDate = date.toString(C_DT_FORMAT);
  QString Date = tr("<center><font size=\"5\">Schedule for ")+selDate+tr(":</font></center>");
  day->setText(Date);
  QString note = tr("<font size=\"5\">Important notes, journal for ")+selDate+tr(":</font>");
  noteLabel->setText(note);
  
}

/*TODO: Fix the BUG: The first characters font after change
  Does not get changed.Don't know why.

*/


void qOrganizer::modifyText()
{
 /*When a new character apears select it and change it's font to the currently selected font*/

 
 cursor = textField->textCursor();
 if(cursor.selectedText()=="")
 {
  cursor.movePosition(QTextCursor::Left,QTextCursor::KeepAnchor);
  //cout << "Selected text is:";
  textField->setCurrentFont(font);
  /*cout << cursor.selectedText()<<"\n";
  cout <<"Font used:";
  cout << font.rawName() <<" "<<font.pointSize()<<"\n";
  */
  cursor.movePosition(QTextCursor::Right);
 };
};

//This function sets the font for the text written in the Jurnal 
void qOrganizer::setFont() //Set the currently selected font
{
 font=fontBox->currentFont();
 font.setPointSize((comboSize->currentText()).toInt());
 
 if(B->isChecked())
   font.setBold(true);
  
 if(I->isChecked()) 
   font.setItalic(true);

 if(U->isChecked()) 
   font.setUnderline(true);
 
 textField->setCurrentFont(font);

 //cout << font.rawName() <<" "<<font.pointSize()<<"\n";
};


//Empty the table
void qOrganizer::emptyTable()
{
    cout <<"Blanking the table\n";
    tableWid->clearContents();  
    tableWid->setRowCount(C_NRROWS);
    tableWid -> setColumnWidth(0,285);
    tableWid -> setColumnWidth(1,60);
    tableWid -> setColumnWidth(2,60);
    tableWid -> setColumnWidth(3,75);  

};

//This function changes to the directory in which the callendar stores it's data
//By default this is ~/.qOrganizer/calendar but I plan to make a setting dialog for this
void qOrganizer::setCalDir()
{
 if(C_PATH=="home")
  QDir::setCurrent(QDir::homePath());
   else QDir::setCurrent(C_PATH);
 QDir currentDir = QDir::current();
 currentDir.mkdir(".qOrganizer");  
 currentDir.cd(".qOrganizer");
 currentDir.mkdir("calendar");
 currentDir.cd("calendar");
 QDir::setCurrent(currentDir.path()); 
 QDate date = calendar->selectedDate();
 selDate = date.toString("MM_dd_yyyy")+".txt";
};


void qOrganizer::loadSchedule() //Load the daily schedule into the schedule table named tableWid
{
 setCalDir();
 QFile scheduleFile(selDate); 
 //For writing QStrings to stdout
 QTextStream cout(stdout, QIODevice::WriteOnly);

 cout <<"Loading file:"<< selDate <<"\n";
 emptyTable();
 if (scheduleFile.open(QFile::ReadOnly)) 
 {
  QTextStream in(&scheduleFile);
  in.setCodec("UTF-8");
  int i=-1;  
  while(!in.atEnd()) 
   {
    QString line=in.readLine(); 
    i++; 
    if(line!="\n")
    {
     QStringList list = line.split("|", QString::KeepEmptyParts);
     if(i>=tableWid->rowCount())
     {
      insertRowToEnd();
      //cout<<"Adding a new row:"<<tableWid->rowCount()<<"\n";
     }
     for (int j = 0; j < list.size(); j++)
      {
       item= new QTableWidgetItem(QTableWidgetItem::Type);
       if(list.at(j)==" ")
        item->setText(""); 
         else 
          item->setText(list.at(j)); 
       //This is the way to access an item from the QStringList
       if(tableWid->item(i,j)!=0) delete tableWid->takeItem(i,j);
       tableWid->setItem(i,j,item);
     };
    }
   }  
   tableWid->setRowCount(i+1);
   scheduleFile.close();
  }
  else 
   {
    emptyTable();  
   }
};

//The tables are stored in a textfile, items are separated with | 

void qOrganizer::saveSchedule() //Save the daily schedule
{
 setCalDir();
 QFile scheduleFile(selDate); 
 //For writing QStrings to stdout
 QTextStream cin(stdin, QIODevice::ReadOnly);
 QTextStream cout(stdout, QIODevice::WriteOnly);
 QTextStream cerr(stderr, QIODevice::WriteOnly);

 cout << "Saving "<<selDate<<"\n";

 if (scheduleFile.open(QFile::WriteOnly)) 
 {
  QTextStream out(&scheduleFile);
  out.setCodec("UTF-8");
  int x=tableWid->currentRow();
  int y=tableWid->currentColumn();  

  for(int i=0;i<tableWid->rowCount();i++)
   { 
    for(int j=0;j<tableWid->columnCount();j++)
     {
      tableWid->setCurrentCell(i,j); 
      item=tableWid->currentItem();
      if(item!=0)
       out << item->text()+"|";
        else out << " |";
    }
     out <<"\n";
   }
  scheduleFile.close();
  tableWid->setCurrentCell(x,y);
 }
 else {
         QMessageBox::warning(NULL, "qOrganizer", "Unable to write schedule file.", "OK");
      }

}
 

//----------------------JURNALS SLOTS FOR CALENDAR--------------------

//This saves the journal into an html file named as jurnal_currentDate.html
void qOrganizer::saveJurnal()
{
 //For writing QStrings to stdout
  QTextStream cin(stdin, QIODevice::ReadOnly);
  QTextStream cout(stdout, QIODevice::WriteOnly);
  QTextStream cerr(stderr, QIODevice::WriteOnly);

 setCalDir();
 QDate date = calendar->selectedDate();
 QString fileName ="jurnal_"+date.toString("MM_dd_yyyy")+".html";
 QFile jurnalFile(fileName); 
 cout <<"Saving journal file:\n";
 cout <<fileName<<"\n";
  if (jurnalFile.open(QFile::WriteOnly)) 
  {
   
   QTextStream out(&jurnalFile);
   out.setCodec("UTF-8");
   QString text = textField->toHtml();
   out << text; 
  }
  else
   {
     QMessageBox::warning(NULL, "qOrganizer", "Unable to write journal file.", "OK");
   }; 
 jurnalFile.close();
};

//This loads the jurnal file to the CQTextEdit object named textField which is the Jurnal editor
void qOrganizer::loadJurnal()
{
 //For writing QStrings to stdout
 QTextStream cin(stdin, QIODevice::ReadOnly);
 QTextStream cout(stdout, QIODevice::WriteOnly);
 QTextStream cerr(stderr, QIODevice::WriteOnly);

 setCalDir();
 cout <<"Loading journal file:\n";
 QDate date = calendar->selectedDate();
 QString fileName ="jurnal_"+date.toString("MM_dd_yyyy")+".html";
 QFile jurnalFile(fileName); 
 cout<<fileName<<"\n";

  if (jurnalFile.open(QFile::ReadOnly)) 
  {
   QTextStream in(&jurnalFile);
   in.setCodec("UTF-8");
   QString text=in.readAll();
   textField->setHtml(text);
  }
  else
   {
    textField->setHtml("");
   };
 jurnalFile.close();
};

//This sets a timer to check for reminders every 30 seconds, this way we can still do the job in a 
//CPU friendly way
void qOrganizer::setReminders()
{
  connect(timer, SIGNAL(timeout()), this, SLOT(remind()));
  timer->start(C_TIMEOUT);
};

//We need this in the remind function
bool CQTableWidget::isBeingEdited()
{
 if(this->state()==QAbstractItemView::EditingState) 
  return 1;
   else return 0;
};

//This checks if it has to remind the user of an event
void qOrganizer::remind()
{
  //This is for debuging purposes only
  QTextStream cout(stdout, QIODevice::WriteOnly); 
  //We remember the coordinates of the current cell to set it back
  int x=tableWid->currentRow();
  int y=tableWid->currentColumn();  
  /*We need to set the calendar to the date of today to load all reminders for today  
   but this could be annoying for the user in a maximized state so we do it only if minimized
   just to be safe
   This is implemented in the reimplementation of hideEvent.
   Everytime the app gets minimized it returns to the current day so reminders apply only for that
   day and nothing is missed.
 */

 //We can't parse the table for data if the user is editing a cell because it would get it all //selected and when the user types a new character it would be overwritten, the reminder column is 
 //an exception because we need to check for errors and report them right away.
 if((!tableWid->isBeingEdited())||(y==3))
 {  
 //Parse the table for reminders
  for(int i=0;i<=tableWid->rowCount();i++)
   {    
       tableWid->setCurrentCell(i,3); 
       item=tableWid->currentItem();
       if(item!=0)
        {  
          QString timeString = item->text();
          tableWid->setCurrentCell(i,0); 
          item=tableWid->currentItem();
          QString eventString;
          if(item!=0)
            eventString = item->text();
             else eventString ="";
           
          //This is because there are to excepted formats for the timeString
          // Example: 12:00 is equal to 12
          if(timeString.indexOf(":")!=-1)
           {
             //Cut the string and make it into a QTime object
             if(timeString.size()==4)
              timeString="0"+timeString;
             QString hourString=timeString;
             hourString.remove(hourString.indexOf(":"),3); 
             
             QString minuteString=timeString;
             minuteString.remove(0,3);
             bool ok;
             bool ok2;
             int hour = hourString.toInt(&ok,10);
             int minute = minuteString.toInt(&ok2,10);
             if((ok&&ok2)&&(hour<24)&&(hour>=0)&&(minute>=0)&&(minute<=60))
             {
              QTime time = QTime(hour,minute,0,0);
              QDate date = calendar -> selectedDate();
              if((time<QTime::currentTime())&&(date==QDate::currentDate()))
               { 
                 if(C_SOUND) playSound(); 
                 tableWid->setCurrentCell(i,3); 
                 item=tableWid->currentItem();
                 item -> setText("");
                 //To avoid phantom reminders
                 saveSchedule();
                 QSystemTrayIcon::MessageIcon icon = QSystemTrayIcon::MessageIcon(QSystemTrayIcon::Information);                
                /*Pops up a baloon in the system tray notifying the user about the event
                 This stays active for 2 hours (7200 seconds), I guess that's enough for the user to
                 notice it.*/
                 if(C_BALOON_SET)
                  tray->showMessage("qOrganizer",eventString,icon,7200000);      
                   else
                      {
                       if(!this->isVisible()) toggleVisibility();
                       QMessageBox::about(this,"qOrganizer", eventString); 
                      }
                      //This will look pretty and not block the sound since it's not modal.
                 tableWid->setCurrentCell(i,3); 
                 item=tableWid->currentItem();
                 item -> setText("");
                 //To avoid phantom reminders
                 saveSchedule();                 
              }
             } 
             else 
              {
               QMessageBox::warning(NULL, "qOrganizer",tr("Couldn't set reminder!\nThere are two excepted formats for reminders.\nOne is hh:mm.\nExample:17:58.\nOr the other one is just writing the hour.\nExample: 3 is equal to 3:00 or to 03:00" ) , "OK");
               tableWid->setCurrentCell(i,3); 
               item=tableWid->currentItem();
               item -> setText("");
               //To avoid phantom reminders
               saveSchedule();
             }
           }
           else
            {
              if((timeString.size()==2)||(timeString.size()==1))
               {
                 bool ok;
                 int hour = timeString.toInt(&ok,10);
                 if((ok)&&(hour<24)&&(hour>=0)) 
                  {
                     QTime time = QTime(hour,0,0,0);
                     QDate date = calendar -> selectedDate();
                     if((time<QTime::currentTime())&&(date==QDate::currentDate()))
                      {                      
                        if(C_SOUND) playSound(); 
                        tableWid->setCurrentCell(i,3); 
                        item=tableWid->currentItem();
                        item -> setText("");
                        //To avoid phantom reminders
                        saveSchedule();
                        QSystemTrayIcon::MessageIcon icon = QSystemTrayIcon::MessageIcon(QSystemTrayIcon::Information);
                        if(C_BALOON_SET)
                         tray->showMessage("qOrganizer",eventString,icon,7200000);     
                         else
                           {
                            if(!this->isVisible()) toggleVisibility();
                            QMessageBox::about(this,"qOrganizer", eventString); 
                           }
                     
                          //This will look pretty and not block the sound since it's not modal.                 
                      }
                  } 
                   else 
                    {
                      if((timeString.size()!=0)&&(timeString!=" ")&&(timeString!="  ")
                                                            &&(timeString!="   ")) 
                       {
                        QMessageBox::warning(NULL, "qOrganizer",tr("Couldn't set reminder!\nThere are two excepted formats for reminders.\nOne is hh:mm.\nExample:17:58.\nOr the other one is just writing the hour.\nExample: 3 is equal to 3:00 or to 03:00" ) , "OK");
                        tableWid->setCurrentCell(i,3); 
                        item=tableWid->currentItem();
                        item -> setText("");          
                        //To avoid phantom reminders
                        saveSchedule();    
                       }
                    } 
                    
               }
                else 
                {
                 if((timeString.size()!=0)&&(timeString!=" ")&&(timeString!="  ")
                                                            &&(timeString!="   ")) 
                  {
                   QMessageBox::warning(NULL, "qOrganizer",tr("Couldn't set reminder!\nThere are two excepted formats for reminders.\nOne is hh:mm.\nExample:17:58.\nOr the other one is just writing the hour.\nExample: 3 is equal to 3:00 or to 03:00" ) , "OK");
                   tableWid->setCurrentCell(i,3); 
                   item=tableWid->currentItem();
                   item -> setText("");        
                   //To avoid phantom reminders
                   saveSchedule();      
                  }
                } 
            }
        };
   };
 tableWid -> setCurrentCell(x,y);
 };
};


//---------------------------------------SIGNALS---------------------------------------

//Reimplementation of focusOutEvent to emit a signal we can catch to know
//when to save
//AUTOSAVE works on focusOutEvents so it consumes less cpu then connecting it to 
//cellChanged(int,int)
void CQTableWidget::focusOutEvent( QFocusEvent * event )
{
 if((event->reason()==Qt::MouseFocusReason) ||
    (event->reason()==Qt::TabFocusReason)||
    (event->reason()==Qt::MenuBarFocusReason)||
    (event->reason()==Qt::ActiveWindowFocusReason))
 {
  emit focusLost();
 };
};

void CQTextEdit::focusOutEvent( QFocusEvent * event )
{
 if((event->reason()==Qt::MouseFocusReason) ||      
    (event->reason()==Qt::TabFocusReason) ||
    (event->reason()==Qt::MenuBarFocusReason) ||
    (event->reason()==Qt::ActiveWindowFocusReason))
 {
  emit focusLost();
 };
};


//------------------------TO-DO LIST SLOTS--------------------------

void qOrganizer::newTask()
{
 list->setRowCount(list->rowCount()+1);
};

void qOrganizer::deleteTask()
{
 list->removeRow(list->currentRow());
};

//Sets the todo list directory 
void qOrganizer::setTDDir()
{
 if(C_PATH=="home")
  QDir::setCurrent(QDir::homePath ());
   else QDir::setCurrent(C_PATH);
 QDir currentDir = QDir::current();
 currentDir.mkdir(".qOrganizer");  
 currentDir.cd(".qOrganizer");
 currentDir.mkdir("todo");
 currentDir.cd("todo");
 QDir::setCurrent(currentDir.path());
};

//Saves the to-do list
void qOrganizer::saveToDoList()
{
 setTDDir();
 QFile todoFile("todo.txt"); 
 cout <<"Saving to-do list file:\n";

  if (todoFile.open(QFile::WriteOnly)) 
  {
   QTextStream out(&todoFile);
   out.setCodec("UTF-8");
   int x=list->currentRow();
   int y=list->currentColumn();  
   for(int i=0;i<list->rowCount();i++)
    { 
     for(int j=0;j<list->columnCount();j++)
      {
       list->setCurrentCell(i,j); 
       item=list->currentItem();
       if(item!=0)
        out << item->text()+"|";
         else out << " |";
      }
     out <<"\n";
     }
   todoFile.close();
   list->setCurrentCell(x,y); 
  }
  else 
   {
     QMessageBox::warning(NULL, "qOrganizer", "Unable to write To-Do list file.", "OK");
   };
};

//Loads the to-do list
void qOrganizer::loadToDoList()
{
  setTDDir();
  QFile todoFile("todo.txt"); 
  if (todoFile.open(QFile::ReadOnly)) 
  {
   cout << "Loading to-do list\n";
   QTextStream in(&todoFile);
   in.setCodec("UTF-8");
   int i=-1;
   while(!in.atEnd()) 
    {
     QString line=in.readLine(); 
     i++; 
     if(line!="\n")
     {
      QStringList slist = line.split("|", QString::SkipEmptyParts);
      if(i>=list->rowCount())
      {
       newTask();
       cout<<"TODO:Adding a new row:"<<list->rowCount()<<"\n";
      }; 
      for (int j = 0; j < slist.size(); j++)
       {
        item= new QTableWidgetItem(QTableWidgetItem::Type);
        item->setText(slist.at(j));
        if(list->item(i,j)!=0) delete list->takeItem(i,j);
        list->setItem(i,j,item);
       };
      }
   }  
   list->setRowCount(i+1);
 }
 todoFile.close();
 list -> setSortingEnabled(true);
};
  

//-------------------------TIMETABLE SLOTS---------------------

void qOrganizer::newRow()
{
 table->setRowCount(table->rowCount()+1);
};

void qOrganizer::delRow()
{
 table->removeRow(table->currentRow());
};

void qOrganizer::delColumn()
{ 
 table->removeColumn(table->currentColumn());
 TTlabels.removeLast();
 englishList.removeLast();
}

//This brings up a dialog and asks for a dayname to add as a column because in some countryes
//they have school even saturday.Doesn't that suck :( ?

void qOrganizer::newColumn()
{
  bool ok;
  QString text = QInputDialog::getText(this,
                                       "",
                                       tr("Column name:"), QLineEdit::Normal,
                                       tr("Saturday"), &ok,
                                       Qt::Dialog
                                      );
  if (!ok || text.isEmpty()) 
    { 
      text=tr("Saturday");
    };
 
 table->setColumnCount(table->columnCount()+1);
 TTlabels<<text;
 englishList << text;
 table->setHorizontalHeaderLabels(TTlabels);
 for(int i=2;i<=table->columnCount()-1;i++)
  table -> setColumnWidth(i,900/(table->columnCount()-1));
 };

//Set's the directory for the Timetable
void qOrganizer::setTTDir()
{
 if(C_PATH=="home")
  QDir::setCurrent(QDir::homePath ());
   else QDir::setCurrent(C_PATH);
 QDir currentDir = QDir::current();
 currentDir.mkdir(".qOrganizer");  
 currentDir.cd(".qOrganizer");
 currentDir.mkdir("timetable");
 currentDir.cd("timetable");
 QDir::setCurrent(currentDir.path());
};

//Saves the timetable
void qOrganizer::saveTimeTable()
{
 setTTDir();
 QFile timeTableFile("timetable.txt"); 
 cout <<"Saving timetable file\n";

  if (timeTableFile.open(QFile::WriteOnly)) 
  {
   QTextStream out(&timeTableFile);
   out.setCodec("UTF-8");
   int x=table->currentRow();
   int y=table->currentColumn();
   //Save column names
   if(!englishList.isEmpty())
    for(int i=0;i<englishList.size();i++)
      out << englishList.at(i)<<"|";
    else 
     {
      for(int i=0;i<universalTTlabels.size();i++)
       out << universalTTlabels.at(i)<<"|";
       englishList = universalTTlabels;
     }
   out<<"\n";  
   for(int i=0;i<table->rowCount();i++)
    { 
     for(int j=0;j<table->columnCount();j++)
      {
       table->setCurrentCell(i,j); 
       QTableWidgetItem *item=table->currentItem();
       if(item!=0)
        out << item->text()+"|";
         else out <<" |";
      }
     out <<"\n";
     }
   timeTableFile.close();
   table->setCurrentCell(x,y); 
  }
  else 
   {
     QMessageBox::warning(NULL, "qOrganizer", "Unable to write timetable file.", "OK");
   };
};

//Loads the timetable
void qOrganizer::loadTimeTable()
{
  setTTDir();
  QFile timeTableFile("timetable.txt"); 
  if (timeTableFile.open(QFile::ReadOnly)) 
  {
   cout << "Loading timetable\n";
   QTextStream in(&timeTableFile);
   in.setCodec("UTF-8");
   int i=-1;
   
   QString firstline=in.readLine(); 
   QStringList labelsList = firstline.split("|", QString::SkipEmptyParts);
   englishList = labelsList;
   for(int i=0;i<labelsList.size();i++)
    {
     QString temp = tr(labelsList.at(i).toUtf8().data());
     labelsList.replace(i,temp);
    }
   table->setColumnCount(labelsList.size()); 
   table->setHorizontalHeaderLabels(labelsList);
   TTlabels = labelsList;

   while(!in.atEnd()) 
    {
     QString line=in.readLine(); 
     i++; 
     if(line!="\n")
     {
      QStringList slist = line.split("|", QString::SkipEmptyParts);
      if(i>=list->rowCount())
      {
       newRow();
       cout<<"TimeTable:Adding a new row:"<<table->rowCount()<<"\n";
      }; 
      for (int j = 0; j < slist.size(); j++)
       {
        item= new QTableWidgetItem(QTableWidgetItem::Type);
        item->setText(slist.at(j));
        if(table->item(i,j)!=0) delete table->takeItem(i,j);
        table->setItem(i,j,item);
       };
      }
   }  
   table->setRowCount(i+1);
 }
 timeTableFile.close();
};

//------------------------------------------BOOKLET SLOTS------------------------------------------

//New Column 
void qOrganizer::newMark()
{
 markTable->setColumnCount(markTable->columnCount()+1);
 markTable->setColumnWidth(markTable->columnCount()-1,50);
};

//New row with custom header
void qOrganizer::newSubject()
{
  bool ok;
  QString text = QInputDialog::getText(this,
                                       "",
                                       tr("Subject name:"), QLineEdit::Normal,
                                       tr(""), &ok,
                                       Qt::Dialog
                                      );
  if (ok || !text.isEmpty()) 
    { 
      markTable->setRowCount(markTable->rowCount()+1);
      absenceTable->setRowCount(markTable->rowCount());
      subjectList<<text;
      markTable->setVerticalHeaderLabels(subjectList);
    };

};

//Remove column
void qOrganizer::deleteMark()
{
  markTable -> removeColumn(markTable->currentColumn());
};

//Remove row, this also affects the absence table and also deletes it from the QStringList called
//subjectList
void qOrganizer::deleteSubject()
{
 int pos = markTable->currentRow();
 markTable -> removeRow(markTable->currentRow());
 absenceTable->removeRow(markTable->currentRow()+1);
 subjectList.removeAt(pos);
};

//New column in the absence table
void qOrganizer::newAbsence()
{
  absenceTable->setColumnCount(absenceTable->columnCount()+1);
  absenceTable->setColumnWidth(absenceTable->columnCount()-1,80);
};

//Remove column from the absence table
void qOrganizer::deleteAbsence()
{
 absenceTable->removeColumn(absenceTable->currentColumn());
}; 

void qOrganizer::setBDir() //set booklet directory
{
 if(C_PATH=="home")
  QDir::setCurrent(QDir::homePath ());
   else QDir::setCurrent(C_PATH);
 QDir currentDir = QDir::current();
 currentDir.mkdir(".qOrganizer");  
 currentDir.cd(".qOrganizer");
 currentDir.mkdir("booklet");
 currentDir.cd("booklet");
 QDir::setCurrent(currentDir.path());
};

//Saves the contents of the markTable into a textfile
//The first line contains the vertical headers (subjects) of the marktable separated with |
//The other lines contain the marks separated with |
void qOrganizer::saveMarksTable()
{
 setBDir();
 QFile markFile("marks.txt"); 
 cout <<"Saving marks file\n";

  if (markFile.open(QFile::WriteOnly)) 
  {
   QTextStream out(&markFile);
   out.setCodec("UTF-8");
   int x=markTable->currentRow();
   int y=markTable->currentColumn();
   //Save row names
   for(int i=0;i<subjectList.size();i++)
    {
      out << subjectList.at(i)<<"|";
    };
   out<<"\n";  
 
   for(int i=0;i<markTable->rowCount();i++)
    { 
     for(int j=0;j<markTable->columnCount();j++)
      {
       markTable->setCurrentCell(i,j); 
       QTableWidgetItem *item=markTable->currentItem();
       if(item!=0)
        out << item->text()+"|";
         else out <<" |";
      }
     out <<"\n";
     }
   markFile.close();
   markTable->setCurrentCell(x,y); 
  }
  else 
   {
     QMessageBox::warning(NULL, "qOrganizer", "Unable to write marks into file.", "OK");
   };

};

/*
  Here I kind of had a problem with QStringList::split, it worked buggy and I couldn't figure it out
  why so I wrote my own using stl strings and vector
*/

void qOrganizer::splitMyString(QString s)
{
    v.clear();
    string str = s.toStdString();
    string delim="|";
    unsigned int offset = 0;
    unsigned int delimIndex = 0;
    
    delimIndex = str.find(delim, offset);

    while (delimIndex != string::npos)
    {
        v.push_back(str.substr(offset, delimIndex - offset));
        offset += delimIndex - offset + delim.length();
        delimIndex = str.find(delim, offset);
    }

    v.push_back(str.substr(offset));
}


//Loads the marks in markTable
//v is the vector in which the string get's splitted
void qOrganizer::loadMarksTable()
{
  setBDir();
  QFile markFile("marks.txt"); 
 /* //For writing QStrings to stdout
  QTextStream cin(stdin, QIODevice::ReadOnly);
  QTextStream cout(stdout, QIODevice::WriteOnly);
  QTextStream cerr(stderr, QIODevice::WriteOnly);*/

  if (markFile.open(QFile::ReadOnly)) 
  {
   cout << "Loading marks!\n";
   QTextStream in(&markFile);
   in.setCodec("UTF-8");
   int i=-1;
   QString firstline=in.readLine(); 
   QStringList labelsList = firstline.split("|", QString::SkipEmptyParts); //Here it works, funny 
   markTable->setRowCount(labelsList.size()); 
   absenceTable->setRowCount(labelsList.size());
   markTable->setVerticalHeaderLabels(labelsList);
   subjectList=labelsList;
   QStringList slist;

   while(!in.atEnd()) 
    {
     QString line=in.readLine(); 
     i++;
     splitMyString(line);
     markTable->setColumnCount(v.size()-1); 
    //sets the Column count to the number of elements in a line of the textfile
     for(int j=0;j<v.size();j++)
      { 
       item= new QTableWidgetItem(QTableWidgetItem::Type);
       item->setText(QString::fromStdString(v[j]));
       if(markTable->item(i,j)!=0) delete markTable->takeItem(i,j);
       markTable->setItem(i,j,item); 
      }
    }  
 }
 markFile.close();
 for(int a=0;a<markTable->columnCount();a++)
        markTable->setColumnWidth(a,50);
};

//Save absences same format absence dates separated with |
//Example: 20.03.2007|13.01.2007|08.05.2007|
void qOrganizer::saveAbsenceTable()
{
 setBDir();
 QFile absenceFile("absences.txt"); 
 cout <<"Saving absences into file\n";

  if (absenceFile.open(QFile::WriteOnly)) 
  {
   QTextStream out(&absenceFile);
   out.setCodec("UTF-8");
   int x=absenceTable->currentRow();
   int y=absenceTable->currentColumn();
      
   for(int i=0;i<absenceTable->rowCount();i++)
    { 
     for(int j=0;j<absenceTable->columnCount();j++)
      {
       absenceTable->setCurrentCell(i,j); 
       QTableWidgetItem *item=absenceTable->currentItem();
       if(item!=0)
        out << item->text()+"|";
         else out <<" |";
      }
     out <<"\n";
     }
   absenceFile.close();
   absenceTable->setCurrentCell(x,y); 
  }
  else 
   {
     QMessageBox::warning(NULL, "qOrganizer", "Unable to write absences into file.", "OK");
   };

};

//Load the absences into absenceTable
void qOrganizer::loadAbsenceTable()
{ 
  setBDir();
  QFile absenceFile("absences.txt"); 
  if (absenceFile.open(QFile::ReadOnly)) 
  {
   cout << "Loading absences!\n";
   QTextStream in(&absenceFile);
   in.setCodec("UTF-8");
   int i=-1;
   QStringList slist;
   while(!in.atEnd()) 
    {
     QString line=in.readLine(); 
     i++;
     splitMyString(line);
     absenceTable->setColumnCount(v.size()-1);
     for (int j = 0; j < v.size(); j++)
       {
        item= new QTableWidgetItem(QTableWidgetItem::Type);
        item->setText(QString::fromStdString(v[j]));
        if(absenceTable->item(i,j)!=0) delete absenceTable->takeItem(i,j);
        absenceTable->setItem(i,j,item);
       };
     }
   }  
 
 absenceFile.close();
 for(int i=0;i<absenceTable->columnCount();i++)
        absenceTable->setColumnWidth(i,80);
     
};

double qOrganizer::calculateRowAverage(int row)
{
   double average; 
   double sum=0; 
   double nr=0;

   for(int j=0;j<markTable->columnCount();j++)
       {
        markTable->setCurrentCell(row,j); 
        QTableWidgetItem *item=markTable->currentItem();
        if(item!=0)
         {
          sum=sum+(item->text()).toFloat();
          if((item->text()).toFloat()!=0) nr++;
         }
       };
   if(nr!=0) average=sum/nr;
    else average=0;

 
 return average;
};

//This recalculates the average of the marks in markTable when the current row changes
//nr=new row, nc=new column, pr=previous row,pc=previous column

void qOrganizer::updateAverage(int nr,int nc,int pr,int pc)
{
 if(nr!=pr)
 {
  
  int x=markTable->currentRow();
  int y=markTable->currentColumn();
  double average = calculateRowAverage(x);
  markTable->setCurrentCell(x,y); 
  //cout<<"Average:"<<average<<"/"<<nr<<"\n";
  averageField->setText(QString::number(average));
 };
};

//This recalculates the total average based on all data
void qOrganizer::updateTotalAverage()
{
  int x=markTable->currentRow();
  int y=markTable->currentColumn();

  double sum=0;
  double nr = markTable->rowCount();

  for(int i=0;i<markTable->rowCount();i++)
  {
   double result = calculateRowAverage(i);
   result=round(result);
   if(result==0) nr--;
   sum=sum+result;
  };
  
  double average;
  if(nr!=0) average=sum/nr;
   else average=0;
  cout << "Total average:"<<sum<<"/"<<nr<<"="<<sum/nr<<"\n";
  totalAverageField->setText(QString::number(average));
  markTable->setCurrentCell(x,y); 
};

//This recounts the absences in absenceTable when that looses focus
void qOrganizer::updateAbsenceNr()
{
 int nr=0;

 int x=absenceTable->currentRow();
 int y=absenceTable->currentColumn();
 
 for(int i=0;i<absenceTable->rowCount();i++)
    { 
     for(int j=0;j<absenceTable->columnCount();j++)
      {
       absenceTable->setCurrentCell(i,j); 
       QTableWidgetItem *item=absenceTable->currentItem();
       if(item!=0)
        {
         if(((item->text()).size()!=0)&&(item->text()!=" ")&&(item->text()!="  ")) nr++;
        }
      }
     }
 
 absenceTable->setCurrentCell(x,y); 
 absenceNrField->setText(QString::number(nr));
};

//Close event, save on close and quit

void qOrganizer::closeEvent(QCloseEvent *e)
{
 if(C_AUTOSAVE_TOGGLE)
 {
 saveSchedule();
 saveJurnal();
 saveToDoList();
 saveTimeTable();
 saveMarksTable();
 saveAbsenceTable();
 };
 writeSettings();
 
 if(C_SYSTRAY)
  { 
   e->ignore();
   hide();
   
   ChangeToCalendar();
   calendar->setSelectedDate(QDate::currentDate());
   updateDay(); 
   loadJurnal();
   loadSchedule();
  }
  else
     {
      e->accept();
      tray->hide();
      qApp->quit();
     }

}

//---------------------------------SYSTEM TRAY------------------------


//Sets the system tray Icon
void qOrganizer::setSTray()
{
 tray = new QSystemTrayIcon( this );
 tray->setIcon ( QIcon(":/images/icon.png") );
 connect(tray, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),
this,SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));
 tray->show();
}

//reimplementation of event
void qOrganizer::trayActivated(QSystemTrayIcon::ActivationReason reason)
{
	if (reason == QSystemTrayIcon::Trigger)
		toggleVisibility();
}

//You guessed it toggles the visibility of the window
void qOrganizer::toggleVisibility()
{
	if(isHidden())
	{
		show();
		if(isMinimized())
		{
			if(isMaximized())
				showMaximized();
			else
				showNormal();
		}
		raise();
		activateWindow();
	}
	else
	{
		hide();
	}
};

//This catches minimize events and saves when the window gets minimized and also hides
//the window entry in the taskbar so the program sits in the system tray
void qOrganizer::hideEvent(QHideEvent * event)
{
 if(this->isMinimized())
  {
   if(C_AUTOSAVE_TOGGLE)
    {
     saveToDoList();
     saveTimeTable();
     saveSchedule();
     saveJurnal();
     saveMarksTable();
     saveAbsenceTable();
    };
    //Set the calendar back to the current day to make sure reminders will work
    ChangeToCalendar();
    calendar->setSelectedDate(QDate::currentDate());
    updateDay(); 
    loadJurnal();
    loadSchedule();
  }
};

//-------------------------Printing----------------------------

//formats a document with the content of the callendar using a hidden QTextEdit widget and prints it
void qOrganizer::printCalendar()
{
  QTextEdit *hidden=new QTextEdit;
  hidden->setParent(this);
  hidden->show();
  //We construct a string in html and we use setHtml with the hidden QTextEdit
  QString text=day->text();
  text.append("<br>");
  //Now for the schedule table 
  
  int x=tableWid->currentRow();
  int y=tableWid->currentColumn();  //We remember the current column to set it back 

  text.append("<table border=2>");
  text.append("<font size=7>");

  text.append("<tr>");

  for(int i=0;i<Clabels.size();i++)
    { 
     text.append("<td>");
     text.append(Clabels.at(i));
     text.append("</td>");
    };

  text.append("</tr>");

  for(int i=0;i<tableWid->rowCount();i++)
   { 
    text.append("<tr>");

    for(int j=0;j<tableWid->columnCount();j++)
     {
      tableWid->setCurrentCell(i,j); 
      item=tableWid->currentItem();
      if(item!=0)
        text.append("<td>"+item->text()+" </td>");
         else
          text.append("<td> </td>");
     }
    text.append(" </tr><br>");
   }

  tableWid->setCurrentCell(x,y);
  text.append("</font></table><br><br>");
  //Jurnal
  text.append(noteLabel->text()+"<br><br><br>");
  text.append(textField->toHtml());

  hidden->setHtml(text);
  QTextDocument *document = hidden->document();
  hidden->hide();
  document->print(printer);
  delete printer;
  delete hidden;
};

//Prints the to-do list under the same mechanism as in printCalendar()
void qOrganizer::printToDoList()
{ 
  QTextEdit *hidden=new QTextEdit;
  hidden->setParent(this);
  hidden->show();
  QString text;
  int x=list->currentRow();
  int y=list->currentColumn();  

  text="<center><font size=\"8\"><b>To-do List</b></font></center><br><br>";

  text.append("<center><table border=\"2\">");
  text.append("<font size=\"7\">");
  text.append("<tr>");

  for(int i=0;i<TDlabels.size();i++)
    { 
     text.append("<td>");
     text.append(TDlabels.at(i));
     text.append("</td>");
    };

  text.append("</tr>");

  for(int i=0;i<list->rowCount();i++)
   { 
    text.append("<tr>");

    for(int j=0;j<list->columnCount();j++)
     {
      list->setCurrentCell(i,j); 
      item=list->currentItem();
      if(item!=0)
        text.append("<td>"+item->text()+" </td>");
         else
          text.append("<td> </td>");
     }
    text.append(" </tr><br>");
   }

  list->setCurrentCell(x,y);
  text.append("</font></table></center><br><br>");  

  hidden -> setHtml(text);
  QTextDocument *document = hidden->document();
  hidden->hide();
  document->print(printer);
  delete printer;
  delete hidden;
};

//Prints the timetable
void qOrganizer::printTimeTable()
{
  QTextEdit *hidden=new QTextEdit;
  hidden->setParent(this);
  hidden->show();
  QString text;
  int x=table->currentRow();
  int y=table->currentColumn();  

  text="<center><font size=\"8\"><b>Timetable</b></font></center><br>";

  text.append("<center><table border=\"1\">");
  text.append("<font size=\"7\">");
  text.append("<tr>");

  for(int i=0;i<TTlabels.size();i++)
    { 
     text.append("<td>");
     text.append(TTlabels.at(i));
     text.append("</td>");
    };

  text.append("</tr>");

  for(int i=0;i<table->rowCount();i++)
   { 
     text.append("<tr>");

    for(int j=0;j<table->columnCount();j++)
     {
      table->setCurrentCell(i,j); 
      item=table->currentItem();
      if(item!=0)
        text.append("<td><b> "+item->text()+"</b></td>");
         else
          {
           text.append("<td> </td>");
          };
     }
    text.append("</tr><br>");
   }

  table->setCurrentCell(x,y);
  text.append("</font></table></center><br><br>");  


  hidden ->setHtml(text);
  
  QTextDocument *document = hidden->document();
  hidden->hide();
  document->print(printer);
  delete printer;
  delete hidden;
};

//Prints the booklet
void qOrganizer::printBooklet()
{
  QTextEdit *hidden=new QTextEdit;
  hidden->setParent(this);
  hidden->show();
  QString text;
 
  //Marks
  int x=markTable->currentRow();
  int y=markTable->currentColumn();  

  text="<center><font size=\"8\"><b>Marks</b></font></center><br>";

  
  text.append("<table border=\"1\">");
  text.append("<font size=\"7\">");


  for(int i=0;i<markTable->rowCount();i++)
   { 
    text.append("<tr><td>");
    text.append(subjectList.at(i));
    text.append("</td>");

    for(int j=0;j<markTable->columnCount();j++)
     {
      markTable->setCurrentCell(i,j); 
      item=markTable->currentItem();
      if(item!=0)
        text.append("<td><b> "+item->text()+"</b></td>");
         else
          {
           text.append("<td> </td>");
          };
     }
    text.append("</tr><br>");
   }

  markTable->setCurrentCell(x,y);
  text.append("</font></table><br>");  
 
 //Absences

  x=absenceTable->currentRow();
  y=absenceTable->currentColumn();  
 
  text.append("<br><br><br><center><font size=\"8\"><b>Absences</b></font></center><br>");

  text.append("<table border=\"1\">");
  text.append("<font size=\"7\">");

  for(int i=0;i<absenceTable->rowCount();i++)
   { 
    text.append("<tr><td>");
    text.append(subjectList.at(i));
    text.append("</td>");

    for(int j=0;j<absenceTable->columnCount();j++)
     {
      absenceTable->setCurrentCell(i,j); 
      item=absenceTable->currentItem();
      if(item!=0)
        text.append("<td><b> "+item->text()+"</b></td>");
         else
          {
           text.append("<td> </td>");
          };
     }
    text.append("</tr><br>");
   }

  absenceTable->setCurrentCell(x,y);
  text.append("</font></table><br>");  

  hidden ->setHtml(text);
  
  QTextDocument *document = hidden->document();
  hidden->hide();
  document->print(printer);
  delete printer;
  delete hidden;
};


//When the print button is hit on the toolbar the app checkes what page is active and prints the
//currently active page
void qOrganizer::printPage()
{
 printer = new QPrinter();
 QPrintDialog printDialog(printer, this);
 if (printDialog.exec() == QDialog::Accepted)
 { 
  if(mainWid->currentIndex()==0) 
   {
    printCalendar();
   }
   else
    if(mainWid->currentIndex()==1)
     { 
      printToDoList();
     } 
     else 
      if(mainWid->currentIndex()==2)
       {
        printTimeTable();
       }
       else
        if(mainWid->currentIndex()==3)
         {
          printBooklet();
         };
 };

};


//-----------------------------------CONFIG FILE ------------------------------------------------

//sets the home directory where the config file is located
//TODO: I think .mkdir for every call is a bad thing.
void qOrganizer::setConfigDir()
{
 QDir::setCurrent(QDir::homePath ());
 QDir currentDir = QDir::current();
 currentDir.mkdir(".qOrganizer");
 currentDir.cd(".qOrganizer");
 QDir::setCurrent(currentDir.path());
};

//This is called if no config file is available, contains default settings
void qOrganizer::writeDefaultConfigFile()
{
 setConfigDir();
 QFile configFile("config.txt"); 
 cout <<"Writing default config file\n";

  if (configFile.open(QFile::WriteOnly)) 
  {
     QTextStream out(&configFile); 
     out.setCodec("UTF-8");
     out << "home" <<"\n"; //The default directory where the config file is located
     out << "30000" <<"\n"; //The value for the timer to check for reminders
     out << "1" <<"\n";  //First day of the week
     out << "ON" <<"\n"; //Autosave toggle
     out << "Baloon" <<"\n"; //Type of reminder
     out << "4" <<"\n";
     out << "dddd MMMM d. yyyy" << "\n";
     out << "OFF" <<"\n";
     out << "OFF" <<"\n";
  } 
   else
    { 
      QMessageBox::warning(NULL, "qOrganizer", "Unable to write config file.", "OK");
    }; 
 readConfigFile();
};

//Reads the config file and sets the config variables which are noted all upercase starting
//with C_ (from CONFIG)
void qOrganizer::readConfigFile()
{
 setConfigDir();
 QFile configFile("config.txt");
 
 if (configFile.open(QFile::ReadOnly))
 {
   QTextStream in(&configFile);
   in.setCodec("UTF-8");
   C_PATH = in.readLine();
   //QTextStream cout(stdout, QIODevice::WriteOnly);
   C_TIMEOUT=(in.readLine()).toInt();
   C_FIRST_DAY_OF_WEEK = (in.readLine()).toInt();
   QString autosaveString = in.readLine();
   if(autosaveString=="ON") 
    C_AUTOSAVE_TOGGLE=true; 
    else C_AUTOSAVE_TOGGLE = false; 
   QString messageString = in.readLine();
   if(messageString=="Baloon")
    C_BALOON_SET = true;
     else C_BALOON_SET = false;
   C_NRROWS = (in.readLine()).toInt();
   C_DT_FORMAT = in.readLine();
   QString ftpString = in.readLine();
   if(ftpString=="ON")
    C_SYNCFTP=1;
     else C_SYNCFTP=0;
   QString loadString = in.readLine(); 
   if(loadString=="ON")
    C_LOAD_ON_CHANGE=1;
     else C_LOAD_ON_CHANGE=0;
 } 
 else writeDefaultConfigFile();
}; 

//Shows the setting dialog
void qOrganizer::showSettings()
{
 settingsDialog = new settings(this);
 settingsDialog -> show();
};


//Saves the config file this is connected to the OK button of the settings dialog
void qOrganizer::saveConfigFile()
{
 setConfigDir();
 QFile configFile("config.txt");
 if (configFile.open(QFile::WriteOnly))
  { 
    QTextStream out(&configFile);
    if(settingsDialog->pathEdit->text() == QDir::homePath())
     out << "home";
     else 
      if(!settingsDialog->pathEdit->text().endsWith(QDir::separator()))
       out << settingsDialog->pathEdit->text()+QDir::separator();
        else
         out << settingsDialog->pathEdit->text();
    out <<"\n";
    out << settingsDialog -> rBox -> value()<<"\n";
    out << settingsDialog -> firstDay -> currentIndex()+1<<"\n";
    if(settingsDialog->autoBox->isChecked())
     out <<"ON";
      else out << "OFF";
    out <<"\n";
    out << settingsDialog->messageCombo->currentText()<<"\n";
    out << settingsDialog->rownrBox->value()<<"\n";
    out << settingsDialog->dateFormatEdit->currentText()<<"\n";

    if(settingsDialog->ftpBox->isChecked())
     out <<"ON"<<"\n";
      else out <<"OFF"<<"\n";

    if(settingsDialog->loadBox->isChecked())
     out <<"ON"<<"\n";
      else out <<"OFF"<<"\n";
  }
  else 
   {
    QMessageBox::warning(NULL, "qOrganizer", "Unable to write config file.", "OK");
   };
 
 //Store the selected language in the registry 
 QSettings settings("qOrganizer", "qOrganizer");
 settings.setValue("language", settingsDialog->langCombo->currentText());
 settings.setValue("host",settingsDialog->hostEdit->text());
 settings.setValue("port",settingsDialog->portBox->value());
 settings.setValue("user",settingsDialog->userEdit->text());
 settings.setValue("passwd",settingsDialog->passEdit->text());
 settings.setValue("path",settingsDialog->ftppathEdit->text());
 if(settingsDialog->trayBox->isChecked())
  settings.setValue("tosystrayonclose",1);
   else settings.setValue("tosystrayonclose",0);
 
 if(settingsDialog->soundBox->isChecked())
  settings.setValue("soundonremind",1);
   else settings.setValue("soundonremind",0);
 
 settingsDialog->close();
 readConfigFile(); //We re-read it only for the C_TIMEOUT variable to take effect instantly
 readSettings(); //For the ftp to take effect instantly
};


//------------------------------------FTP SYNC------------------------------------------------

/*Ok so this uploads a whole folder to a remote ftp server
The path is the folders name and the dir path is the folder where you keep it
Example: if the folder is /home/user/.qOrganizer then that is the path
and the dirPath is /home/user
This function is called from putToFTP() 

So QFtp is asynchronous, that's a good thing for the user, and bad for us, but we will deal with it.This is pretty readable and simple code:
*/
void qOrganizer::sendFolder(const QString &path,const QString &dirPath)
{
    QTextStream cout(stdout, QIODevice::WriteOnly);
    QFileInfo fileInfo(path);
    if (fileInfo.isDir())
    {
        QString dirName = path;
        dirName.remove(0,dirPath.size());
        if(!noFolders) ftp->mkdir(dirName);
        QDir dir(path);
        QStringList entries = dir.entryList(QDir::AllEntries|QDir::NoDotAndDotDot,QDir::DirsFirst);
        foreach (QString entry, entries)
        {
         statusBar()->showMessage(tr("Uploading..."),10000);
         sendFolder(dir.filePath(entry),dirPath);
        }
    }
    else
    {   
        QFile *file = new QFile(path);
        uploadedFileQueue.enqueue(file);
        if(file->open(QIODevice::ReadOnly))
         {
          QString filename =path;
          filename.remove(0,dirPath.size());
          putCommandVector.append(ftp->put(file,filename));
          cout << "put "<<path<<" "<<filename<<"\n";
          statusBar()->showMessage(tr("Uploading..."),10000);
          file->close();
         }
         else cout <<"Could not open file:"<<path<<"\n";
    }
}


/*
 This is where the fun begins and things get really asynchronous, if you thought the last one was hard to understand then prepare to read some docs.
 We managed to upload a folder. Now we need to download it.
 path is the folders name, and localPath the folder that contains our folder.
 ex: if the folder is /home/user/ftp/.qOrganizer then the path variable is .qOrganizer and the localPath is /home/user/ftp/
The list command emits a signal called listInfo which collects the entryes/
*/
void qOrganizer::getFolder(QString path,QString localPath)
{
 QTextStream cout(stdout, QIODevice::WriteOnly);
 QDir localDir(localPath);
 listCommandVector.append(ftp->list(path)); //we track the commands in vectors
 stack.push(path); 
 localDir.mkdir(path);
 connect(ftp,SIGNAL(listInfo(QUrlInfo)),this,SLOT(processFolder(QUrlInfo)));
 connect(ftp,SIGNAL(commandFinished(int,bool)),this,SLOT(processCommand(int,bool)));
 ftpEntryes = new QStringList; //this will contain the file list
 lpath=localPath;
}

//This makes us a QList with all entryes, folders first then files.

void qOrganizer::processFolder(const QUrlInfo &i)
{
 if(i.isDir())
  {
   entryList.push_front(i); 
  }
   else 
     if(i.isFile()) 
      entryList.push_back(i);     
}


static int nrcomfin=0; 
//nr of finished command, this counts how many times this function is called
void qOrganizer::processCommand(int com,bool err)
{
 //Variables
 nrcomfin++;
 QString path;
 QTextStream cout(stdout, QIODevice::WriteOnly);
 bool containsSubfolders=0;
 bool entryExists=0;
 QDir localDir(lpath);
 //If the command was list() then we start procesing our QList
 if(listCommandVector.indexOf(com)!=-1)
  { 
   path = stack.top();
   foreach(QUrlInfo entry,entryList)
    {
      if((entry.isFile())&&(listedFiles.indexOf(entry)==-1))
       {
        QString foldPath = path+"/"+entry.name();
        ftpEntryes->append(foldPath);
        listedFiles.append(entry);
        //cout <<"File:"<<foldPath<<"\n";
        entryExists=1;
       }
       else
        if((entry.isDir())&&(listedFolders.indexOf(entry)==-1))
         {
          QString foldPath = path+"/"+entry.name();
          //cout <<"Folder:"<<foldPath<<"\n";
          stack.push(foldPath); 
          localDir.mkdir(foldPath);
          //cout << stack.top() <<"\n";
          listCommandVector.append(ftp->list(foldPath));
          listedFolders.append(entry);
          containsSubfolders=1;
          entryExists=1;
          break;
         }
    }
   if((stack.size()>1)&&(nrcomfin%2!=0)) stack.pop();
   entryList.clear();
   //cout << stack.size() << stack.top() <<" "<<nrcomfin<<"\n"; 
   if((!containsSubfolders)&&(entryExists)) 
      listCommandVector.append(ftp->list(stack.top()));
   if((!containsSubfolders) && (!entryExists)) downloadFiles();   
  }
  else //If the command was a get
   if(getCommandVector.indexOf(com)!=-1)
    {
     ftpFile=fileQueue.dequeue();
     ftpFile-> close();
     //cout <<"Completed:"<<ftpFile->fileName()<<" "<<"Errors:"<<err<<"\n";
     ftpFile->deleteLater();
    }
    else
     if(putCommandVector.indexOf(com)!=-1)
      {
       QFile *myFile = uploadedFileQueue.dequeue();
       myFile->deleteLater();
      };
}

void qOrganizer::downloadFiles()
{
 QTextStream cout(stdout, QIODevice::WriteOnly);
 //cout <<"Downloading"<<"\n";
 foreach(QString entry,*ftpEntryes)
 {
  QFile *file = new QFile(lpath+entry);
  cout << "get " << entry <<" "<<lpath+entry<<"\n";
  statusBar()->showMessage(tr("Downloading..."),10000);
  fileQueue.enqueue(file);
  file->open(QIODevice::WriteOnly);
  getCommandVector.append(ftp -> get(entry,file));
 }
}

void qOrganizer::connectToFTP()
{
 ftp = new QFtp(this);
 ftp -> connectToHost(C_HOST,C_PORT);
 ftp -> login(C_USER,C_PASSWD);
 ftp -> cd(C_FTPPATH);
}

//Two simplyfied aliases

void qOrganizer::putToFTP()
{ 
 statusProgress -> show();
 connectToFTP();
 uploadAct->setEnabled(false);
 connect(ftp,SIGNAL(done(bool)),this,SLOT(updateStatusP(bool)));
 if(C_PATH=="home")  
  sendFolder(QDir::toNativeSeparators(QDir::homePath()+"/")+".qOrganizer",QDir::toNativeSeparators(QDir::homePath()+"/"));
   else 
     sendFolder(C_PATH+".qOrganizer",C_PATH);
}

void qOrganizer::getFromFTP()
{
 statusProgress -> show();
 connectToFTP();
 downloadAct->setEnabled(false);
 if(C_AUTOSAVE_TOGGLE) toggleAutoSave(0);
 connect(ftp,SIGNAL(done(bool)),this,SLOT(updateStatusG(bool)));
 if(C_PATH=="home")  
  getFolder(QString(".qOrganizer"),QDir::toNativeSeparators(QDir::homePath()+"/"));
   else 
    getFolder(QString(".qOrganizer"),C_PATH);
}

void qOrganizer::updateStatusG(bool error)
{
 downloadAct->setEnabled(true);
 statusProgress -> hide();
 if(!this->isVisible()) toggleVisibility();
 if(!error) 
  {
   statusBar()->showMessage(tr("Done!"),10000);
   QMessageBox::warning(NULL,tr("Done"),tr("Download finished!"));
   
   loadToDoList();
   loadTimeTable();
   loadSchedule();
   loadJurnal();
   loadMarksTable();
   loadAbsenceTable();
    
    ftp->deleteLater();
  }
   else
    {
      QMessageBox::warning(NULL, "Error",tr("Couldn't download files:\n")+ftp->errorString());
      ftp->deleteLater();
    }
  disconnect(ftp, SIGNAL(done(bool)),this,SLOT(updateStatusG(bool)));
  if(changedAutoSave) 
   { 
    toggleAutoSave(1); 
   }
  stack.clear();
  delete ftpEntryes;
  entryList.clear();
  listedFiles.clear();
  listedFolders.clear();
  listCommandVector.clear();
  getCommandVector.clear();
  nrcomfin=0;
}

void qOrganizer::updateStatusP(bool error)
{
 QTextStream cout(stdout, QIODevice::WriteOnly);
 statusProgress -> hide();
 cout << " Upload finished:"<<error<<" "<<ftp->errorString()<<"\n";
 uploadAct->setEnabled(true);
 if(!this->isVisible()) toggleVisibility();
 if(!error) 
  {
   statusBar()->showMessage(tr("Done!"),10000);
   QMessageBox::warning(NULL,tr("Done"),tr("Upload finished!"));
   ftp->deleteLater();
   noFolders=false;
  }
   else
    {
     if((ftp->errorString().indexOf(".qOrganizer")!=-1)&&(ftp->errorString().indexOf("exists")!=-1))
      {
       noFolders=true;
       ftp->deleteLater();
       disconnect(ftp, SIGNAL(done(bool)),this,SLOT(updateStatusP(bool)));
       cout <<"Error encountered:Folder exists, entering noFolder mode and reuploading\n";
       //Don't know why we need to call it twice, but it doesn't work if it is called only once.
       //Maybe one gets killed by deleteLater
       putToFTP();
       putToFTP();
      }
       else
        {
          QMessageBox::warning(NULL,tr("Error"),QString(tr("Couldn't upload files:\n")+ftp->errorString()));
          ftp->deleteLater();
        }
    }
 disconnect(ftp, SIGNAL(done(bool)),this,SLOT(updateStatusP(bool)));
}

//if ind is true turn it on else turn it off
void qOrganizer::toggleAutoSave(bool ind)
{
 if(!ind)
  {
   disconnect(tableWid,SIGNAL(focusLost()),this,SLOT(saveSchedule()));
   disconnect(textField,SIGNAL(focusLost()),this,SLOT(saveJurnal()));
   disconnect(list,SIGNAL(focusLost()),this,SLOT(saveToDoList()));
   disconnect(table,SIGNAL(focusLost()),this,SLOT(saveTimeTable()));
   disconnect(markTable,SIGNAL(focusLost()),this,SLOT(saveMarksTable()));
   disconnect(absenceTable,SIGNAL(focusLost()),this,SLOT(saveAbsenceTable()));
   changedAutoSave=1;
  }
  else
   {
    connect(tableWid,SIGNAL(focusLost()),this,SLOT(saveSchedule()));
    connect(textField,SIGNAL(focusLost()),this,SLOT(saveJurnal()));
    connect(list,SIGNAL(focusLost()),this,SLOT(saveToDoList()));
    connect(table,SIGNAL(focusLost()),this,SLOT(saveTimeTable()));
    connect(markTable,SIGNAL(focusLost()),this,SLOT(saveMarksTable()));
    connect(absenceTable,SIGNAL(focusLost()),this,SLOT(saveAbsenceTable()));
    changedAutoSave=0;
   }
};




//Needed by custom delegate to be able to delete from the absenceTable, all because there is no 
//way to create an empty QDateTimeEdit
//We previously called setObjectName for these tables
void CQTableWidget::keyPressEvent (QKeyEvent * event)
{
 if(this->objectName()==QString("absenceTable")) //The table with the absences
 {
   if(event->key() == Qt::Key_Backspace || event->key() == Qt::Key_Delete)
   {
       this->currentItem()->setText(" ");
       return;
   }
 }
 else
  if(this->objectName()==QString("list")) //To-do list table
   { 
     if((this->currentColumn()==2)||(this->currentColumn()==0))
      {
       if(event->key() == Qt::Key_Backspace || event->key() == Qt::Key_Delete)
       {
        this->currentItem()->setText(" ");
        return;
       }
      }
   }
   else
    if(this->objectName()==QString("tableWid")) //Schedule table
     {
      if(this->currentColumn()!=0)
      {
       if(event->key() == Qt::Key_Backspace || event->key() == Qt::Key_Delete)
       {
        this->currentItem()->setText(" ");
        return;
       }
      } 
     }
 return QTableWidget::keyPressEvent(event);
}

void qOrganizer::saveAll()
{
 saveToDoList();
 saveTimeTable();
 saveSchedule();
 saveJurnal();
 saveMarksTable();
 saveAbsenceTable();
}

void qOrganizer::exitApp()
{
 if(C_AUTOSAVE_TOGGLE)
 {
 saveSchedule();
 saveJurnal();
 saveToDoList();
 saveTimeTable();
 saveMarksTable();
 saveAbsenceTable();
 };
 writeSettings();

 qApp->quit();
}

void qOrganizer::playSound()
{
 if(QSound::isAvailable())
  {
   QFile soundFile(":images/sound.wav");
   if(C_PATH=="home")
    {
     soundFile.copy(QDir::homePath()+QDir::separator()+".qOrganizer"+QDir::separator()+"sound.wav");
     QSound::play(QDir::homePath()+QDir::separator()+".qOrganizer"+QDir::separator()+"sound.wav");
    }
     else   
      {
       soundFile.copy(C_PATH+".qOrganizer"+QDir::separator()+"sound.wav");
       QSound::play(C_PATH+".qOrganizer"+QDir::separator()+"sound.wav");
      }
  }
  else 
   {//Linux uses NAS, which kind of sucks with qt, so we have to do this primitively
    #ifdef Q_OS_LINUX
     QFile soundFile(":images/sound.wav");
     if(!soundFile.copy("/tmp/qremindersound.wav")) 
      cout << "Could not copy soundfile to /tmp\n";
     system("cat /tmp/qremindersound.wav >> /dev/dsp"); 
    #endif
   }
} 

qOrganizer::~qOrganizer()
{
}


